@@ -1,3 +1,4 @@
+
 ///////////////////////////////////////////////////////////////////////////
 // Copyright (C) 2008  Whit Armstrong                                    //
 //                                                                       //
@@ -34,6 +35,7 @@
 #include <tslib/utils/breaks.hpp>
 #include <tslib/vector.transform.hpp>
 #include <tslib/date.policies/posix.date.policy.hpp>
+#include <tslib/date.policies/julian.date.policy.hpp>
 #include <tslib/date.policies/date.partition.hpp>
 
 namespace tslib {
@@ -49,23 +51,20 @@ namespace tslib {
             template<typename> class DatePolicy>
   class TSeries {
   private:
-    TSDATABACKEND<TDATE,TDATA,TSDIM>* tsdata_;
+    TSDATABACKEND<TDATE,TDATA,TSDIM> tsdata_;
     const TSDIM offset(const TSDIM row, const TSDIM col) const {
       return row + col*nrow();
     }
   public:
     // ctors dtors
-    ~TSeries();
     TSeries();
-    TSeries(TSDATABACKEND<TDATE,TDATA,TSDIM>* tsdata);
+    TSeries(TSDATABACKEND<TDATE,TDATA,TSDIM>& tsdata);
     TSeries(const TSDIM rows, const TSDIM cols);
     TSeries(const TSeries& T);
-
-    // copy an object
-    TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> copy() const;
+    TSeries(TDATA* external_data, TDATE* external_dates, const TSDIM nrows, const TSDIM ncols, const bool release);
 
     // accessors
-    TSDATABACKEND<TDATE,TDATA,TSDIM>* getIMPL() const;
+    const TSDATABACKEND<TDATE,TDATA,TSDIM>* getIMPL() const;
     std::vector<std::string> getColnames() const;
     const TSDIM nrow() const;
     const TSDIM ncol() const;
@@ -111,27 +110,43 @@ namespace tslib {
     const TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> diff(const unsigned int n) const;
 
     // matix index
-    TDATA operator() (const TSDIM row, const TSDIM col) const;
+    const TDATA operator() (const TSDIM row, const TSDIM col) const;
     TDATA& operator() (const TSDIM row, const TSDIM col);
 
     friend std::ostream& operator<< <> (std::ostream& os, const TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>& ts);
   };
 
+  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
+  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries() {}
+
+  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
+  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries(TSDATABACKEND<TDATE,TDATA,TSDIM>& tsdata): tsdata_(tsdata) {}
+
+  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
+  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries(const TSDIM rows, const TSDIM cols): tsdata_(rows,cols) {}
+
+  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
+  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries(const TSeries& T): tsdata_(T.tsdata_) {}
+
+  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
+  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries(TDATA* external_data, TDATE* external_dates, const TSDIM nrows, const TSDIM ncols, const bool release):
+    tsdata_(external_data, external_dates, nrows, ncols, release) {}
+
+  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
+  inline
+  const TSDATABACKEND<TDATE,TDATA,TSDIM>* TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::getIMPL() const {
+    return &tsdata_;
+  }
+
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>& TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::operator=(const TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>& rhs) {
 
     // self assignment
-    if(tsdata_ == rhs.tsdata_) {
+    if(this == &rhs) {
       return *this;
     }
-
-    // release old data
-    tsdata_->detach();
-
-    // attach to new data
+    // assign to new data
     tsdata_ = rhs.tsdata_;
-    tsdata_->attach();
-
     return *this;
   }
 
@@ -156,11 +171,7 @@ namespace tslib {
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   const TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::lag(const unsigned int n) const {
-    if(n >= nrow()) {
-      TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> bad_ans(0,ncol());
-      bad_ans.setColnames(getColnames());
-      return bad_ans;
-    }
+    if(n >= nrow()) { throw std::logic_error("lag: n > nrow of time series."); }
     const TSDIM new_size = nrow() - n;
     TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> ans(new_size, ncol());
     TDATA* ans_data = ans.getData();
@@ -182,11 +193,7 @@ namespace tslib {
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   const TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::lead(const unsigned int n) const {
-    if(n >= nrow()) {
-      TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> bad_ans(0,ncol());
-      bad_ans.setColnames(getColnames());
-      return bad_ans;
-    }
+    if(n >= nrow()) { throw std::logic_error("lead: n > nrow of time series."); }
     const TSDIM new_size = nrow() - n;
     TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> ans(new_size, ncol());
     TDATA* ans_data = ans.getData();
@@ -208,11 +215,7 @@ namespace tslib {
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   const TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::diff(const unsigned int n) const {
-    if(n >= nrow()) {
-      TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> bad_ans(0,ncol());
-      bad_ans.setColnames(getColnames());
-      return bad_ans;
-    }
+    if(n >= nrow()) { throw std::logic_error("diff: n > nrow of time series."); }
     const TSDIM new_size = nrow() - n;
     TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> ans(new_size, ncol());
     TDATA* ans_data = ans.getData();
@@ -240,8 +243,8 @@ namespace tslib {
 
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
-  TDATA TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::operator() (const TSDIM row, const TSDIM col) const {
-    TDATA* data = getData();
+  const TDATA TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::operator() (const TSDIM row, const TSDIM col) const {
+    const TDATA* data = getData();
     return data[offset(row, col)];
   }
 
@@ -281,84 +284,41 @@ namespace tslib {
     return os;
   }
 
-  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
-  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::~TSeries() {
-    tsdata_->detach();
-  }
-
-  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
-  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries() {
-    tsdata_ = TSDATABACKEND<TDATE,TDATA,TSDIM>::init();
-  }
-
-  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
-  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries(TSDATABACKEND<TDATE,TDATA,TSDIM>* tsdata) {
-    tsdata_ = tsdata;
-  }
-
-  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
-  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries(const TSDIM rows, const TSDIM cols) {
-    tsdata_ = TSDATABACKEND<TDATE,TDATA,TSDIM>::init(rows,cols);
-  }
-
-  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
-  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::TSeries(const TSeries& T) {
-    tsdata_ = T.tsdata_;
-    tsdata_->attach();
-  }
-
-  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
-  TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy> TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::copy() const {
-    TSeries ans(nrow(),ncol());
-    ans.setColnames(getColnames());
-
-    std::copy(getDates(),getDates()+nrow(),ans.getDates());
-    std::copy(getData(),getData()+nrow()*ncol(),ans.getData());
-
-    return ans;
-  }
-
-  template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
-  inline
-  TSDATABACKEND<TDATE,TDATA,TSDIM>* TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::getIMPL() const {
-    return tsdata_;
-  }
-
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   inline
   TDATE* TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::getDates() const {
-    return tsdata_->getDates();
+    return tsdata_.getDates();
   }
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   inline
   TDATA* TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::getData() const {
-    return tsdata_->getData();
+    return tsdata_.getData();
   }
 
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   inline
   const TSDIM TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::nrow() const {
-    return tsdata_->nrow();
+    return tsdata_.nrow();
   }
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   inline
   const TSDIM TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::ncol() const {
-    return tsdata_->ncol();
+    return tsdata_.ncol();
   }
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   inline
   std::vector<std::string> TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::getColnames() const {
-    return tsdata_->getColnames();
+    return tsdata_.getColnames();
   }
 
   template<typename TDATE, typename TDATA, typename TSDIM, template<typename,typename,typename> class TSDATABACKEND, template<typename> class DatePolicy>
   int TSeries<TDATE,TDATA,TSDIM,TSDATABACKEND,DatePolicy>::setColnames(const std::vector<std::string>& cnames) {
     if(static_cast<TSDIM>(cnames.size()) == ncol()) {
-      tsdata_->setColnames(cnames);
+      tsdata_.setColnames(cnames);
       return 0; // SUCCESS
     } else {
       return 1; // FAILURE
